function [Kc,Kf,Q,M]=cor2obr(plant,SYS_Kr,tol)
%
%==========================================================
% Observer-Based Realization of a given controller
%==========================================================
%
% [KC,KF,Q,M] = COR2OBR(PLANT,SYS_K) compute a real Observer
% Based Realization, that is the Youla parameterization
% (defined by Kc, Kf and Q), of a given continuous-time
% controller SYS_K for a given continous-time plant 
% PLANT in the case:
%  NK (SYS_K order) >= N (PLANT order): 
%
% Remarks: * SYS_K, PLANT and Q are defined as SS object,
%          * feedback between PLANT and SYS_K is positive,
%          * a real solution may not exit,
%          * NQ (order of Q) = NK - N.
% 
% This function plots the map of closed-loop eigenvalues (red x)
% and PLANT open-loop eigenvalues (blue +) in the complex plane. 
% Then, the user can choose, in a interactive procedure, the 
% closed-loop  eigenvalues distribution between:
%  * state feedback dynamics [A-BKc] (blue o),
%  * state estimation dynamics [A-KfC] (red o),
%  * Youla parameter dynamics (Q)  (green o).
%
% Uncontrollable eigenvalues are automatically assigned to [A-BKc].
% Unobservable eigenvalues are automatically assigned to [A-KfC].
% (the controller SYS_K is assumed to be minimal).
% Auto-conjugate eigenvalues are assigned together.
%
% M is the transformation matrix between the old and the new state
% space realizations of the controller:
%         X_k = M [X_hat X_Q].
%
% [KC,KF,Q,M] = COR2OBR(PLANT,SYS_K,TOL) allows a tolerance TOL
% (default: 10^-6) to be taken into account in the unobservable
% and uncontrollable subspaces computation.
%
% Reference: D. Alazard and P. Apkarian "Exact observer based structures
% for arbitrary compensators" International Journal of Robust and 
% Non-Linear Control, 1999, n.9, pp 101-118 
%
% See also: COR2OBRA, OBR2COR, COR2TFG, COR2TFGA

% D. Alazard 01/99
% revised: 11/06
% revised: 11/10 to handle SS objects
% Copyright (c) 1993-2000 ONERA/DCSD, All Rights Reserved.

% Closed-loop matrix computation:

if nargin==2, tol=10^-6;end;
[ac,bc,cc,dc]=ssdata(SYS_Kr);
[A,B2,C2,D22]=ssdata(plant);
n=size(A,1);nk=size(ac,1);ntot=n+nk;
if nk<n, 
    disp('Controller order lower than plant model order: no solution !!');
    Kc=[];Kf=[];Q=[];M=[];
    return;
end
[acd,bcd,ccd,dcd]=feedback(ac,bc,cc,dc,[],[],[],-D22);
[abf,bbf,cbf,dbf]=feedback(A,B2,C2,0*D22,acd,bcd,ccd,dcd,1);

% ===================================================
% Closed-loop and open loop eigen values map 
% ===================================================
[V,D]=eig(abf);
lesvp=diag(D);
nbreal=find(imag(lesvp)==0);
if (mod(n,2)~=0),  % case n is odd
    if (length(nbreal)==0),  %  ===> no solution
        disp('No real solution !!');
        Kc=[];Kf=[];Q=[];M=[];
        return
    end
    if (nk>n)&(length(nbreal))<2,
        disp('No real solution !!');
        Kc=[];Kf=[];Q=[];M=[];
        return
    end
end
figure
plot(real(lesvp),imag(lesvp),'rx','LineWidth',2);
hold on
vpG=eig(A);
plot(real(vpG),imag(vpG),'b+','LineWidth',2);
sgrid
zoom on

% ==================================================
% Interactive selection
% ==================================================
nbf=length(lesvp);
nbo=size(A,1);
index1=[];
index2=[];
reste=[];
ii=1;
while ii<=nbf,
  valp=lesvp(ii);
  if imag(valp)==0;
      ir=ii;ii=ii+1;
  else
      ir=[ii ii+1];ii=ii+2;
  end
  if rank([A-valp*eye(nbo,nbo) B2],tol)<nbo,
      index1=[ir index1];
      if length(index1)>nbo, disp('No solution !!!');return;end;
      plot(real(lesvp(ir)),imag(lesvp(ir)),'bo','LineWidth',2);
  elseif rank([A'-valp*eye(nbo,nbo) C2'],tol)<nbo,
      index2=[index2 ir];
      if length(index2)>nbo, disp('No solution !!!');return;end;
      plot(real(lesvp(ir)),imag(lesvp(ir)),'ro','LineWidth',2);
  else
      reste=[reste ir];
  end
end
index3=[];
ii=1;
while ii<=length(reste),
   valp=lesvp(reste(ii));
   if imag(valp)==0;
      ir=reste(ii);ii=ii+1;
   else
      ir=[reste(ii) reste(ii+1)];ii=ii+2;
   end
   ok=0;
   while ~ok
      text=['Where do you want to assign: ',...
             num2str(valp),' ?'];
      choix=menu(text,'A-BKc','A-KfC','Q');
      switch choix,
      case 1, 
         if length(index1)==nbo,
             disp('A-BKc dynamics is fulfilled, choose another one !');
         elseif length(index1)+length(ir)>nbo
             disp('Only a real eigenvalue can be now assigned to A-BKc!!');
         else
             ok=1;index1=[index1 ir];
             plot(real(lesvp(ir)),imag(lesvp(ir)),'bo','LineWidth',2);
         end
      case 2,
         if length(index2)==nbo,
             disp('A-KfC dynamics is fulfilled, choose another one !');
         elseif length(index2)+length(ir)>nbo
             disp('Only a real eigenvalue can be now assigned to A-KfC!');
         else
             ok=1;index2=[ir index2];
             plot(real(lesvp(ir)),imag(lesvp(ir)),'ro','LineWidth',2);
        end
      case 3,
         if length(index3)==nbf-nbo-nbo,
             disp('Q(s) dynamics is fulfilled, choose another one !');
         elseif length(index3)+length(ir)>nbf-nbo-nbo,
             disp('Only a real eigenvalue can be now assigned to Q(s) !');
         else
             ok=1;index3=[index3 ir];
             plot(real(lesvp(ir)),imag(lesvp(ir)),'go','LineWidth',2);
         end
      end
   end
end

index132=[index1 index3 index2];
M=V(:,index132);
D=D(index132,index132);
[M,D]=cdf2rdf(M,D);

% ==================================================
% Oberver-based parameter computation
% ==================================================


T=real(M(nbo+1:nbf,1:nbo)*inv(M(1:nbo,1:nbo)));

F=acd-T*B2*ccd;
G=bcd-T*B2*dcd;

U=real(T*M(1:nbo,n+1:nbf)-M(nbo+1:nbf,nbo+1:nbf));
U1=U(:,1:nbf-2*nbo);
Um1=inv(U);
U1t=Um1(1:nbf-2*nbo,:);
U2t=Um1(nbf-2*nbo+1:nbf-nbo,:);

Gtilde1=U1t*G;
Gtilde2=U2t*G;
Ttilde1=U1t*T;
Ttilde2=U2t*T;

Kf=inv(Ttilde2)*Gtilde2;
Kc=-ccd*T-dcd*C2;
AQ=real(D(nbo+1:nbf-nbo,nbo+1:nbf-nbo));
BQ=Gtilde1-Ttilde1*inv(Ttilde2)*Gtilde2;
CQ=ccd*U1;
DQ=dcd;

Q=ss(AQ,BQ,CQ,DQ);
  
% matrice de passage finale:
M=[T U1];

