function [COR] = obr2cor2ddl(plant,K,G,Q,H,M);
% [COR]=OBR2COR2DDL(PLANT,K,G,Q,H) 
% compute the state space realization SYS_K=ss(AK,BK,CK,DK) 
% of the 2 degree-of-freedom observer-based controller defined by the 
% Youla parameterization:
%    - K: state feedback gain, 
%    - G: state estimation gain, 
%    - Q(s): dynamic Youla parameter,
%  (this parameterization concern the plant defined by PLANT)
%    - H: a static feedforward matrix (H=eye(size(K,1)) by default).
%
% Inputs of SYS_K are measurement (y) and reference input (r).
%
% PLANT,Q and SYS_K are defined as SS objects.
%
% Let us note: [A,B,C,D]=ssdata(PLANT), then the controller 
% realization is:
%                 ________________________
%                |                        |
%   xHatDot      | A-BK-GC+GDK | G | B-GD |              xHat
%         u <----|     -K      | 0 |   I  |<-----   _     y 
%    y-yhat /----|    -C+DK    | I |  -D  |<---0<--|H|--- r
%           |    |________________________|  + |+   -
%           |                                  |
%           |               ______             |
%           |              |      |            |
%           \------------->| Q(s) |------------/
%                          |______|
%
% Remark :  * continuous-time formulation (or predictor form in
%             discrete-time) for the state estimator
%           * positive feeedback
%
% See also: COR2OBR, COR2OBRA, OBR2COR
% 
% Other syntax:
% [SYS_K]=REGLQGQ(PLANT,K,G,Q,H,M) allows to take into account a 
%   static input matrix M=[M1;M2] between H.r and  xHatDot (through
%  matrix M1) and xQdot (through matrix M2).

% D. Alazard 01/99
% Copyright (c) 1993-2000 ONERA/DCSD, All Rights Reserved.

    
[A,B,C,D] = ssdata(plant);
n=size(A,1);
[p,m]=size(D);
if nargin<4, Q=ss(zeros(m,p));end;
[Aq,Bq,Cq,Dq] = ssdata(Q);
nq=size(Aq,1);
if nargin<5, H=eye(m);end;
if nargin<6, M=zeros(n+nq,m); end;

 w = inv(eye(size(Dq*D))+Dq*D);

 Ccorr = -w*[K+Dq*C -Cq];

 Dcorr = w*[Dq H];

 Acorr = [A-G*C 0*ones(n,nq);-Bq*C Aq]+[B-G*D;-Bq*D]*Ccorr; 
 
 Bcorr = [[G;Bq] M*H]+[B-G*D;-Bq*D]*Dcorr;

 COR = ss(Acorr,Bcorr,Ccorr,Dcorr);          
